from typing import Optional

import cv2
from jvi.geo.size2d import Size
from jvi.image.image_nda import ImageNda, ImageNdas
from jvi.image.proc import tile


def trace_image(image: ImageNda, title: Optional[str] = None, delay: int = 0, box_size: Optional[Size] = None,
                auto_close: bool = True) -> tuple[int, ImageNda]:
    """跟踪显示图片，自动缩放"""
    assert isinstance(image, ImageNda)

    src_size = image.size()
    title = title or str(image)
    if box_size:
        dst_size = src_size.scale_in(box_size).round()
        img_dst = cv2.resize(image.data(), dst_size.to_tuple_int())
    else:
        img_dst = image.data()
    cv2.imshow(title, img_dst)
    key = cv2.waitKey(delay)
    if auto_close:
        cv2.destroyWindow(title)
    assert isinstance(key, int)
    return key, ImageNda(data=img_dst)


def trace_images(images: ImageNdas, title: Optional[str] = None, delay: int = 0, box_size: Size = Size(),
                 auto_close: bool = True) -> tuple[int, ImageNda]:
    """跟踪显示多张图片(并列)"""
    image = tile(images, box_size=box_size)
    return trace_image(image, title, delay, box_size, auto_close)


def close_all_windows() -> None:
    """关闭所有窗口"""
    cv2.destroyAllWindows()



#!/opt/ias/env/bin/python

import argparse
import glob
import os
import sys

import cv2
import matplotlib
import matplotlib.pyplot as plt
from interval import Interval

from jcx.ui.key import Key


def main():
    parser = argparse.ArgumentParser(description='图像工具箱')
    parser.add_argument('source', type=str, help='数据源，文件/目录')
    parser.add_argument('--img-size', type=int, default=224, help='输入图像尺寸')
    parser.add_argument('-r', '--aspect-ratio', nargs=2, type=float, default=[0.0, 100.0], help='纵横比范围')
    parser.add_argument('-s', '--show-hist', action='store_true', help='显示直方图')
    parser.add_argument('-v', '--verbose', action='store_true', help='显示详细信息')
    opt = parser.parse_args()

    if os.path.isdir(opt.source):
        s = os.path.join(opt.source, "**/*.jpg")
        files = sorted(glob.glob(s, recursive=True))
    elif os.path.isfile(opt.source):
        files = [opt.source]
    else:
        print("数据源不存在")
        sys.exit(0)

    radio_range = Interval(opt.aspect_ratio[0], opt.aspect_ratio[1])

    print(radio_range)

    ratio_list = []

    for i, f in enumerate(files):
        img = ImageNda.load(f)  # BGR
        if img is None:
            # print(i, f + '\tERROR')
            continue

        ratio = img.shape[0] / img.shape[1]
        if ratio not in radio_range:
            continue

        ratio_list.append(ratio)
        print(f)

        if opt.verbose:
            cv2.imshow(opt.source, img)
            if cv2.waitKey(0) == Key.ESC.value:
                break
    if opt.show_hist:
        matplotlib.use('TkAgg')
        plt.hist(ratio_list, bins=100, facecolor="blue", edgecolor="black")
        plt.xlabel("aspect ratio")
        plt.ylabel("amount")
        plt.title("aspect ratio hist")
        plt.show()


if __name__ == '__main__':
    main()



import itertools

import cv2  # type: ignore
import numpy as np
from jvi.drawing.color import Color, Colors
from jvi.geo.point2d import array_is_normalized, array_absolutize, Points, Point
from jvi.geo.rectangle import Rect
from jvi.geo.size2d import Size
from jvi.image.image_nda import ImageNda
from typing import Optional


def put_text(canvas: ImageNda, text: str, left_bottom: Point, color: Color, thickness: int = 1,
             scale: float = 0.7) -> None:
    """显示文字"""
    cv2.putText(canvas.data(), text, left_bottom.to_tuple_int(), 0, scale, color.bgr(), thickness, lineType=cv2.LINE_AA)


def rectangle(canvas: ImageNda, rect: Rect, color: Color, thickness: int = 1) -> None:
    """绘制矩形"""
    if rect.is_normalized():
        rect = rect.absolutize(canvas.size())

    p1, p2 = rect.ltrb_tuple_int()
    cv2.rectangle(canvas.data(), p1, p2, color.bgr(), thickness)


def points_to_contour(points: Points):
    """点集转轮廓"""
    pts = []
    for p in points:
        pts.append([p.x, p.y])
    contours = np.array(pts, np.int32)
    return contours.reshape((-1, 1, 2))


def in_polygon(p: Point, polygon: Points) -> int:
    """判定点在多边形内"""

    contour = points_to_contour(polygon)
    return cv2.pointPolygonTest(contour, (p.x, p.y), False)


def line(canvas: ImageNda, p1: Point, p2: Point, color: Color, thickness: int = 1) -> None:
    """绘制直线"""
    canvas_size = canvas.size()
    if p1.is_normalized():
        p1 = p1.absolutize(canvas_size)
    if p2.is_normalized():
        p2 = p2.absolutize(canvas_size)

    cv2.line(canvas.data(), p1.to_tuple_int(), p2.to_tuple_int(), color.bgr(), thickness)


def polylines(canvas: ImageNda, points: Points, color: Color, thickness: int = 1) -> None:
    """绘制多边形"""

    if array_is_normalized(points):
        points = array_absolutize(points, canvas.size())

    contours = points_to_contour(points)
    cv2.polylines(canvas.data(), [contours], True, color.bgr(), thickness)


# [opencv-python3 | cv2.findContours()检测图像中物体轮廓](https://blog.csdn.net/Just_do_myself/article/details/124215020)


def cross(canvas: ImageNda, center: Point, radius: int, color: Color, thickness: int = 1) -> None:
    """绘制十字"""

    if center.is_normalized():
        center = center.absolutize(canvas.size())

    p1 = center + Point(-radius, 0)
    p2 = center + Point(radius, 0)
    cv2.line(canvas.data(), p1.to_tuple_int(), p2.to_tuple_int(), color.bgr(), thickness)

    p1 = center + Point(0, -radius)
    p2 = center + Point(0, radius)
    cv2.line(canvas.data(), p1.to_tuple_int(), p2.to_tuple_int(), color.bgr(), thickness)


def grid(canvas: ImageNda, tile_size: Size, color: Optional[Color] = None, thickness: int = 1,
         colors: Colors = []) -> None:
    """绘制网格"""

    canvas_size = canvas.size()
    if tile_size.is_normalized():
        tile_size = tile_size.absolutize(canvas_size)

    colors1 = itertools.cycle([color] if color else colors)
    for x in range(0, int(canvas_size.width), int(tile_size.width)):
        color1 = next(colors1)
        p1 = Point(x, 0)
        p2 = Point(x, canvas_size.height)
        line(canvas, p1, p2, color1, thickness)

    colors1 = itertools.cycle([color] if color else colors)
    for y in range(0, int(canvas_size.height), int(tile_size.height)):
        color1 = next(colors1)
        p1 = Point(0, y)
        p2 = Point(canvas_size.width, y)
        line(canvas, p1, p2, color1, thickness)



from typing import Optional

import cv2  # type: ignore
from numpy import ndarray

from jvi.geo.point2d import Point
from jvi.geo.size2d import Size
from jvi.gui.image_viewer import ImageViewer


class VideoViewer(ImageViewer):
    """视频查看窗口"""

    def __init__(self, url, title: str, size: Size = Size()):
        super().__init__(title, size)  # REMARK：用URL可能因为汉字等字符报错
        self.__frames = {}
        self.__capture = cv2.VideoCapture(str(url))

        w = int(self.__capture.get(cv2.CAP_PROP_FRAME_WIDTH))
        h = int(self.__capture.get(cv2.CAP_PROP_FRAME_HEIGHT))
        self.__image_size = Size(w, h)

        assert self.image_count() > 0
        self.change_background()

    def is_opened(self) -> bool:
        """判定是否成功打开"""
        return self.__capture.isOpened()

    def image_size(self) -> Size:
        """获取视频图片尺寸"""
        return self.__image_size

    def on_draw(self, canvas: ndarray, pos: Point):
        """设置窗口重绘事件响应"""
        # self.record().draw_on(canvas, pos)
        pass

    def image_at(self, index: int) -> Optional:
        """获取指定索引的图片"""

        print('index:', index)

        frame = self.__frames.get(index)
        if frame is None:
            pos = int(self.__capture.get(cv2.CAP_PROP_POS_FRAMES))
            assert pos <= index
            while pos <= index:
                ok, frame = self.__capture.read()
                if not ok:
                    break
                self.__frames[pos] = frame
                pos += 1
            frame = self.__frames.get(index)

        return frame

    def image_count(self) -> int:
        """获取图片总数，注意：实际帧数可能该值"""
        return int(self.__capture.get(cv2.CAP_PROP_FRAME_COUNT))


def main():
    file = 'file:///home/jiang/ws/lift/2021-08-19/1.mp4'
    # file = '/home/jiang/ws/nvidia/nano_test/data/1.h264'

    win = VideoViewer(file, size=Size(1280, 720))
    print("frame count:", win.image_count())
    print('video size:', win.image_size())
    win.run()


if __name__ == '__main__':
    main()



#!/opt/ias/env/bin/python
import argparse

import sh


# 参考: http://amoffat.github.io/sh/

def main():
    parser = argparse.ArgumentParser('ffmpeg文件分割工具')  # TODO
    parser.add_argument('src_dir', type=Path, help='文件来源目录')
    parser.add_argument('-e', '--ext', type=str, default='jpg', help='文件扩展名')
    parser.add_argument('-m', '--mate_ext', type=str, default='lbl', help='文件伙伴文件扩展名')
    parser.add_argument('-i', '--invert_match', action='store_true', help='选中不匹配的文件')
    parser.add_argument('-v', '--verbose', action='store_true', help='显示详细信息')
    opt = parser.parse_args()

    mates = files_in(opt.src_dir, opt.mate_ext)
    names = [f.stem for f in mates]


if __name__ == '__main__':
    main()



import re
from copy import copy
from dataclasses import dataclass
from typing import Optional, Final, Self

from jcx.m.number import real_2d, Real1_2D, Real

from jvi.geo import is_normalized
from jvi.geo.point2d import Points, Point


@dataclass
class Size:
    """二维尺寸"""
    width: float = 0
    height: float = 0

    def size(self) -> Self:
        """实现ISize2D"""
        return self

    def __bool__(self) -> bool:
        """判定是否存在非零属性"""
        return self != Size()

    def brief(self) -> str:
        """获取简要描述"""
        return "{}x{}".format(self.width, self.height)

    def area(self) -> Real:
        """获取面积"""
        return self.width * self.height

    def empty(self) -> bool:
        """判定面积为零"""
        return self.area() == 0

    def positive(self) -> bool:
        """判定长宽为正"""
        return self.width > 0 and self.height > 0

    def center(self) -> Point:
        """获取中心坐标"""
        return Point(x=self.width / 2, y=self.height / 2)

    def aspect_ratio(self) -> float:
        """纵横比"""
        return self.height / self.width

    def contains(self, other: Self) -> bool:
        """是否包含目标尺寸"""
        return self.width >= other.width and self.height >= other.height

    def clone(self) -> Self:
        """克隆对象"""
        return copy(self)

    def set(self, other: Self) -> None:
        """设置为目标对象值"""
        self.width = other.width
        self.height = other.height

    def scale_me(self, n: Real1_2D) -> None:
        """缩放指定的倍数"""
        x, y = real_2d(n)
        self.width *= x
        self.height *= y

    def scale(self, n: Real1_2D) -> Self:
        """缩放指定的倍数"""
        s = self.clone()
        s.scale_me(n)
        return s

    def to_tuple(self) -> tuple[float, float]:
        """获取tuple"""
        return self.width, self.height

    def to_tuple_int(self) -> tuple[int, int]:
        """转换为tuple"""
        return int(self.width), int(self.height)

    def to_shape(self) -> tuple[int, int]:
        """转换为 2D Shape"""
        return int(self.height), int(self.width)

    def to_shape3d_i(self, channel: int) -> tuple[int, int, int]:
        """转换为 3D Shape交错格式"""
        return int(self.height), int(self.width), channel

    def to_shape3d_p(self, channel: int) -> tuple[int, int, int]:
        """转换为 3D Shape平坦格式"""
        return channel, int(self.height), int(self.width)

    def to_point(self) -> Point:
        """转换为Point"""
        return Point(x=self.width, y=self.height)

    def normalize_me(self, size: Self) -> None:
        """绝对坐标归一化"""
        self.width /= size.width
        self.height /= size.height

    def normalize(self, size: Self) -> Self:
        """获取绝对坐标归一化"""
        p = copy(self)
        p.normalize_me(size)
        return p

    def absolutize_me(self, size: Self) -> None:
        """归一化坐标绝对化"""
        self.width *= size.width
        self.height *= size.height
        self.round_me()  # 存在非整数可能

    def absolutize(self, size: Self) -> Self:
        """获取归一化坐标绝对化"""
        p = copy(self)
        p.absolutize_me(size)
        return p

    def round_me(self) -> None:
        """近似成整数"""
        self.width = round(self.width)
        self.height = round(self.height)

    def round(self) -> Self:
        """近似成整数"""
        s = copy(self)
        s.round_me()
        return s

    def is_normalized(self) -> bool:
        """判断坐标是否被归一化"""
        return is_normalized(self.width) and is_normalized(self.height)

    def __mul__(self, s: Self) -> Self:
        """尺寸相乘"""
        return Size(self.width * s.width, self.height * s.height)

    def __truediv__(self, s: Self) -> Self:
        """尺寸相除"""
        return Size(self.width / s.width, self.height / s.height)

    def __or__(self, s: Self) -> Self:
        """尺寸并集"""
        return Size(max(self.width, s.width), max(self.height, s.height))

    def __and__(self, s: Self) -> Self:
        """尺寸并集"""
        return Size(min(self.width, s.width), min(self.height, s.height))

    def scale_in(self, box_size: Self) -> Self:
        """保证宽高比缩放到指定尺寸以内"""
        rx = box_size.width / self.width
        ry = box_size.height / self.height
        r = min(rx, ry)

        w = self.width * r
        h = self.height * r
        return Size(w, h)


def absolutize_points(points: Points, size: Size) -> Points:
    """点集归一化坐标绝对化"""
    arr = [p.absolutize(size) for p in points]
    return arr


# 参考：https://zh.wikipedia.org/wiki/%E6%98%BE%E7%A4%BA%E5%88%86%E8%BE%A8%E7%8E%87%E5%88%97%E8%A1%A8

SIZE_8K: Final = Size(8192, 4320)
SIZE_8K_UHD: Final = Size(7680, 4320)
SIZE_DCI_4K: Final = Size(4096, 2160)
SIZE_4K_UHD: Final = Size(3840, 2160)
SIZE_3K: Final = Size(2880, 1620)
SIZE_2K: Final = Size(2048, 1080)
SIZE_FHD: Final = Size(1920, 1080)
SIZE_qHD: Final = Size(960, 540)
SIZE_HD_PLUS: Final = Size(1600, 900)
SIZE_HD: Final = Size(1280, 720)
SIZE_nHD: Final = Size(640, 360)
SIZE_VGA: Final = Size(640, 480)
SIZE_QVGA: Final = Size(320, 240)
SIZE_PAL: Final = Size(768, 576)
SIZE_IM: Final = Size(224, 224)

SIZE_640: Final = Size(640, 640)
"""深度学习检测器常规尺寸"""
SIZE_720: Final = Size(720, 720)
"""深度学习检测器常规尺寸"""
SIZE_1280: Final = Size(1280, 1280)
"""深度学习检测器常规尺寸"""

resolutions: Final = {
    '8K': SIZE_8K,
    '8K_UHD': SIZE_8K_UHD,
    'DCI_4K': SIZE_DCI_4K,
    '4K_UHD': SIZE_4K_UHD,
    '3K': SIZE_3K,
    '2K': SIZE_2K,
    'FHD': SIZE_FHD,
    'qHD': SIZE_qHD,
    'HD+': SIZE_HD_PLUS,
    'HD': SIZE_HD,
    'nHD': SIZE_nHD,
    'VGA': SIZE_VGA,
    'QVGA': SIZE_QVGA,
    'PAL': SIZE_PAL,
    'IM': SIZE_IM,
}


def size_parse(s: str) -> Optional[Size]:
    """解析字符串获取尺寸"""

    size = resolutions.get(s)
    if size:
        return size

    p = re.compile(r"(\d+)x(\d+)")
    m = p.match(s.lower())
    if m:
        w = int(m.groups()[0])
        h = int(m.groups()[1])
        size = Size(w, h)
    else:
        size = None
    return size



#!/opt/ias/env/bin/python

import argparse
from pathlib import Path

import cv2  # type: ignore


def main():
    parser = argparse.ArgumentParser(description='图片模糊工具')
    parser.add_argument('src_dir', type=Path, help='来源图片目录')
    parser.add_argument('dst_dir', type=Path, help='目的图片目录')
    parser.add_argument('level', type=int, help='视频模糊级别')
    parser.add_argument('-p', '--pattern', type=str, default='*.jpg', help='图片文件模式(*.jpg)')
    parser.add_argument('-t', '--tail', type=str, default='_b', help='图片后缀')
    parser.add_argument('-v', '--verbose', action='store_true', help='显示详细信息')
    opt = parser.parse_args()

    files = opt.src_dir.rglob(opt.pattern)
    print('源图片目录：%s' % opt.src_dir)

    n = 2 * opt.level + 1
    ksize = (n, n)

    for i, f in enumerate(files):
        src = ImageNda.load(str(f))  # BGR
        if src is None:
            print(i, f.name, '\tERROR')
            continue
        if opt.verbose:
            print(i, f.name, '\tOK')
        dst = cv2.blur(src, ksize)

        name = f.stem + opt.tail + str(opt.level) + f.suffix
        file = opt.dst_dir / name
        print(file)
        cv2.imwrite(str(file), dst)
        # cv2.imshow('src', src)
        # cv2.imshow('dst', dst)
        # cv2.waitKey(0)

    # cv2.destroyAllWindows()


if __name__ == '__main__':
    main()



from copy import copy
from dataclasses import field, dataclass
from typing import Self

from jvi.geo.point2d import Point, Points


@dataclass
class LineSegment:
    """线段"""
    start: Point = field(default_factory=Point)
    end: Point = field(default_factory=Point)

    def clone(self) -> Self:
        """克隆对象"""
        return copy(self)

    def cross(self, p: Point):
        """跨立实验"""
        p1 = self.end - self.start
        p2 = p - self.start
        return p1.x * p2.y - p2.x * p1.y

    def intersects(self, other) -> bool:
        """判断线段是否与另一相交"""
        c1 = self.cross(other.start) * self.cross(other.end)
        c2 = other.cross(self.start) * other.cross(self.end)
        return c1 <= 0 and c2 <= 0

    def intersects_polyline(self, polyline: Points) -> bool:
        """判断线段是否与折线相交"""
        for i in range(len(polyline) - 1):
            p1 = polyline[i]
            p2 = polyline[i + 1]
            if self.intersects(LineSegment(p1, p2)):
                return True
        return False


def line_segment(x1, y1, x2, y2) -> LineSegment:
    """从坐标值构造"""
    return LineSegment(Point(x1, y1), Point(x2, y2))


type LineSegments = list[LineSegment]



#!/opt/ias/env/bin/python

import argparse
from pathlib import Path

from jvi.image.image_nda import ImageNda
from jvi.image.stat import value_map
from jvi.image.trace import trace_image

epilog = '''
Examples:

    jvi-imp src.png dst.png
    
'''


def main():
    parser = argparse.ArgumentParser(description='图片处理工具', epilog=epilog)
    parser.add_argument('src', type=Path, help='来源图片')
    parser.add_argument('dst', type=Path, help='目的图片')
    parser.add_argument('-s', '--size', type=str, help='画布尺寸')
    parser.add_argument('-v', '--verbose', action='store_true', help='显示详细信息')
    opt = parser.parse_args()

    src_img = ImageNda.load(opt.src)

    print('source image shape:', src_img.shape())

    trace_image(src_img)
    m = value_map(src_img)
    print(f'{src_img} hist:')
    for v, p in m.items():
        print(f'\t{v}\t{int(p * 100)}%')


if __name__ == '__main__':
    main()



#!/opt/ias/env/bin/python

import argparse
from pathlib import Path

import sh
from jcx.sys import fs


def main() -> None:
    parser = argparse.ArgumentParser('批量转码工具')
    parser.add_argument('src_dir', type=Path, help='文件来源目录')
    parser.add_argument('dst_dir', type=Path, help='文件目标目录')
    parser.add_argument('-e', '--src_ext', type=str, default='.mp4', help='来源视频文件扩展名')
    parser.add_argument('-d', '--dst_ext', type=str, default='.mp4', help='目标视频文件扩展名')
    parser.add_argument('-V', '--vcodec', type=str, default='h264', help='视频编码: h264/h265')
    parser.add_argument('-a', '--acodec', type=str, default=None, help='音频编码codec')
    parser.add_argument('-s', '--subtitle_ext', type=str, default='.srt', help='字幕文件扩展名')
    parser.add_argument('--crop', type=str, default=None, help='裁切尺寸, 如: 960:720')
    parser.add_argument('-v', '--verbose', action='store_true', help='显示详细信息')
    opt = parser.parse_args()

    files = fs.files_in(opt.src_dir, opt.src_ext)

    print('src files:', len(files))

    vcodec_opts = {
        'h264': ['-c:v', 'h264_nvenc'],
        'h265': ['-vcodec', 'hevc']
    }
    vcodec_opt = vcodec_opts[opt.vcodec]

    ff = sh.Command('/usr/local/bin/ffmpeg')
    for i, src in enumerate(files):
        print(f'#{i + 1} {src} ...')
        sub = src.with_suffix(opt.subtitle_ext)
        dst = opt.dst_dir / (src.stem + opt.dst_ext)
        if dst.is_file():
            print(f'  SKIP: {dst}')
            continue
        args = ['-i', src]
        if sub.is_file():
            args.extend(['-i', sub])
        if opt.crop:
            args.extend(['-vf', f'crop={opt.crop}'])
        args.extend(vcodec_opt)
        if opt.acodec:
            args.extend(['-acodec', opt.acodec])
        args.append(dst)
        ff(*args)


if __name__ == '__main__':
    main()



from dataclasses import dataclass
from pathlib import Path
from typing import Optional

import numpy as np
from arrow import Arrow
from jvi.geo.size2d import Size
from jvi.image.image_nda import ImageNda


@dataclass
class ImageFactory:
    """抓图配置"""

    origin: ImageNda
    """原始图片"""
    path: Optional[Path] = None
    """图片路径"""
    gray: Optional[ImageNda] = None
    """对应的灰度图"""

    def size(self) -> Size:
        """获取图像原始尺寸"""
        return self.origin.size()

    def rgb_data(self) -> np.ndarray:
        """获取图像RGB顺序的数据"""
        return self.origin.data()[:, :, ::-1]  # BGR => RGB


@dataclass
class ImageFactorySet:
    """图片工厂集合"""

    factories: list[Optional[ImageFactory]]
    """图片工厂数组"""
    time: Arrow

    def file_at(self, i: int) -> Path:
        """指定索引的文件"""
        fac = self.factories[i]
        assert fac
        f = fac.path
        assert f
        return f

    def image_at(self, i: int) -> ImageNda:
        """指定索引的图片"""
        fac = self.factories[i]
        assert fac
        im = fac.origin
        assert im
        return im



import cv2
from jcx.ui.key import Key
from jvi.drawing.color import RED, YOLO_GRAY
from jvi.drawing.shape import polylines
from jvi.geo.point2d import Point
from jvi.geo.size2d import Size
from jvi.gui.image_win import ImageWin
from jvi.image.image_nda import ImageNda


class ImpWin(ImageWin):
    """图像处理窗口"""

    def __init__(self, title: str, size: Size):
        super().__init__(title, size)
        self.points = []

    def on_key(self, key: int):
        if key == Key.ESC or key & 0xFF == ord("q"):
            return 1
        elif key == Key.BACKSPACE:
            if self.points:
                self.points.pop()
        elif key > 0:
            print('key:', key)

    def on_draw(self, canvas: ImageNda, _pos: Point):
        color = RED
        if self.points:
            polylines(canvas.data(), self.points, color, 2)

    def on_left_button_down(self, p: Point, flags: int):
        # print(flags)
        self.points.append(p)
        if flags & cv2.EVENT_FLAG_CTRLKEY:
            print('CTRL')
        if flags & cv2.EVENT_FLAG_ALTKEY:
            print('ALT')
        if flags & cv2.EVENT_FLAG_SHIFTKEY:
            print('SHIFT')
        # cv2.setWindowTitle()


def main():
    file = './black_flower.jpg'
    image = ImageNda.load(file)
    size = Size(1280, 720)
    win = ImpWin(file, size)

    win.set_backcolor(YOLO_GRAY)
    win.set_background(image)
    print(win.size())
    win.run()


if __name__ == '__main__':
    main()



from collections.abc import Iterable

from jvi.geo.point2d import Point, Points
from jvi.geo.size2d import Size
from rustshed import Result, Err, Ok


def to_point(p: Point | Size | list | tuple) -> Result[Point, str]:
    """类型转换成Point，接受类型"""
    if isinstance(p, Point):
        return Ok(p)
    if isinstance(p, Size):
        return Ok(Point(p.width, p.height))
    if (isinstance(p, list) or isinstance(p, tuple)) and len(p) == 2:
        return Ok(Point(p[0], p[1]))
    return Err('[ERR] type %s cannot convert to Point' % type(p))


def to_points(points: Iterable) -> Result[Points, str]:
    """转换成点集"""
    if not isinstance(points, Iterable):
        return Err('[ERR] type %s is not iterable' % type(points))
    ps = []
    for p in points:
        p = to_point(p)
        if p.is_err():
            return p
        ps.append(p.unwrap())
    return Ok(ps)


def to_size(s: Size | Point | list | tuple) -> Result[Size, str]:
    """类型转换成Point，接受类型"""
    if isinstance(s, Size):
        return Ok(s)
    if isinstance(s, Point):
        return Ok(Size(s.x, s.y))
    if (isinstance(s, list) or isinstance(s, tuple)) and len(s) == 2:
        return Ok(Size(s[0], s[1]))
    return Err('[ERR] type %s cannot convert to Size' % type(s))



from typing import Optional

from jvi.geo.rectangle import Rect
from jvi.geo.size2d import Size, SIZE_HD
from jvi.image.image_nda import ImageNda, ImageNdas
from jvi.image.trace import trace_image


class TileSet:
    """图片分割成的瓦片集合"""

    def __init__(self, tile_size: Size = Size(), rows: int = 0, cols: int = 0, size: Size = Size(),
                 image: Optional[ImageNda] = None):
        """构造瓦片集合"""
        if tile_size and rows * cols:
            size = Size(tile_size.width * cols, tile_size.height * rows)
        size = size or image.size()
        tile_size = tile_size or Size(size.width / cols, size.height / rows)
        assert tile_size and size

        self._image: ImageNda = image or ImageNda(size)
        rs = Rect.from_size(size).to_tiles(size=tile_size)
        self._tiles: ImageNdas = [self._image.roi(r) for r in rs]

    def __len__(self) -> int:
        """获取瓦片数量"""
        return len(self._tiles)

    def size(self) -> Size:
        """获取图片尺寸"""
        return self._image.size()

    def tile_size(self) -> Size:
        """获取瓦片尺寸"""
        return self._tiles[0].size()

    def image(self) -> ImageNda:
        """获取图片"""
        return self._image

    def tiles(self) -> ImageNdas:
        """获取全部瓦片"""
        return self._tiles


def show_tile():
    from jvi.drawing.color import COLORS7

    size = Size(30, 20)
    tile_size = Size(10, 10)

    ts = TileSet(tile_size=tile_size, size=size)
    assert ts.size() == size
    assert ts.tile_size() == tile_size

    ts = TileSet(rows=2, cols=3, size=size)
    assert ts.size() == size
    assert ts.tile_size() == tile_size

    for i, im in enumerate(ts.tiles()):
        im.set_to(COLORS7[i])

    trace_image(ts.image(), box_size=SIZE_HD)


if __name__ == '__main__':
    show_tile()



from dataclasses import dataclass
from typing import Optional

import numpy as np
from jvi.geo.rectangle import Rect
from jvi.geo.size2d import Size
from jvi.image.image_nda import ImageNda, ImageNdas
from jvi.image.proc import to_gray, merge, split
from jvi.image.tile_set import TileSet


# args = /home/jiang/ws/scene/cnooc/sources/31010102100700101 -v

@dataclass
class ImageItem:
    """图像条目"""
    image: ImageNda
    gray: ImageNda

    def channel_roi(self, channel: int, rect: Rect) -> ImageNda:
        """获取指定通道的指定区域"""
        return self.image.channel_at(channel).roi(rect)

    def roi(self, rect: Rect) -> ImageNda:
        """获取指定区域"""
        return self.image.roi(rect)


CHANNEL_NUM = 3


class Sc3cImageFactory:
    """同色三通道图像工厂"""

    def __init__(self):
        self._size = Size()
        self._items: list[ImageItem] = []
        self._tile3: Optional[TileSet] = None

    def __len__(self) -> int:
        """获取图片数量"""
        return len(self._items)

    def empty(self):
        return len(self) == 0

    def ready(self):
        return len(self) >= CHANNEL_NUM

    def size(self) -> Size:
        return self._size

    def images(self) -> ImageNdas:
        """获取全部图片"""
        return [item.image for item in self._items]

    def push(self, image: ImageNda):
        """压入图片, 图片超过3张, 第一张丢弃"""
        image = image.clone()
        item = ImageItem(image, to_gray(image))

        size = image.size()
        if size != self._size:
            self._size = size
            self._items = []
        if self.ready():
            self._items.pop(0)
        self._items.append(item)

    def sc3c_images(self, rect: Rect = Rect.zero()) -> ImageNdas:
        """获取同色三通道图像:B3,G3,R3,Gray3"""
        rect = rect if rect else Rect.from_size(self.size())
        assert len(self._items) == CHANNEL_NUM
        images = []
        for i in range(CHANNEL_NUM):
            channels = [item.channel_roi(i, rect) for item in self._items]
            images.append(merge(channels))
        channels = [item.gray.roi(rect) for item in self._items]
        images.append(merge(channels))
        return images

    def t3_image(self, rect: Rect = Rect.zero()) -> ImageNda:
        """获取3个源图像上指定区域, 合并成1x3(块)图像"""
        rect = rect if rect else Rect.from_size(self.size())
        assert len(self._items) == CHANNEL_NUM

        if self._tile3 is None or self._tile3.tile_size() != rect.size():
            self._tile3 = TileSet(tile_size=rect.size(), cols=3, rows=1)

        for i, tile in enumerate(self._tile3.tiles()):
            self._items[i].roi(rect).copy_to(tile)
        return self._tile3.image()


def a_test():
    f = '/home/jiang/ws/scene/cnooc/samples_i3/0/31010102100700101_003_2022-11-15_11-05-51.943.jpg'
    im = ImageNda.load(f)
    cs = split(im)
    print(im.shape(), cs[0].shape())
    print(im.data()[:, :, 0:1].shape)
    print(im.data()[..., 0].shape)
    print(im.data()[..., 1].shape)


def a_test1():
    a3d = np.array([
        [[10, 20, 30], [10, 20, 30]],
        [[10, 20, 30], [10, 20, 30]]], dtype=np.uint8
    )
    print(a3d)
    print('shape:', a3d.shape)
    print('p00:', a3d[0, 0])
    print('c0:', a3d[..., 1])

    fac = Sc3cImageFactory()
    fac.push(a3d)
    fac.push(a3d)
    fac.push(a3d)
    ims = fac.sc3c_images(Rect(0, 0, 2, 1))

    for i, im in enumerate(ims):
        print('sc3cim%d' % i, im.shape, ':\n', im)
    ts = fac.t3_image(Rect(0, 0, 2, 2))
    print('tile1x3', ts.shape, ':\n', ts)

    b = 10
    g = 20
    r = 30
    gray = 0.30 * r + 0.59 * g + 0.11 * b
    print('gray:', gray)


if __name__ == '__main__':
    # a_test1()
    a_test()
    # a_mark()
    # mask_test()



#!/opt/ias/env/bin/python

import argparse
from pathlib import Path

from jcx.text.txt_json import load_json, save_json
from jvi.drawing.color import COLORS7
from jvi.drawing.shape import grid
from jvi.geo.size2d import size_parse
from jvi.image.tile import PanoramaInfo, divide_sparse
from jvi.image.io import trace_image, save_image


def main():
    parser = argparse.ArgumentParser(description='瓦片全景查看工具')
    parser.add_argument('panorama', type=Path, help='匹配全景文件')
    parser.add_argument('-n', '--no-background', action='store_true', help='禁用背景图片')
    parser.add_argument('-d', '--divide-size', type=str, help='切分尺寸，比如：4x4')
    parser.add_argument('-o', '--output', type=Path, help='图输出文件/目录')
    parser.add_argument('-g', '--grid_thickness', type=int, default=None, help='绘制网格的线宽')
    parser.add_argument('-v', '--verbose', action='store_true', help='显示详细信息')
    opt = parser.parse_args()

    pano_info = load_json(opt.panorama, PanoramaInfo)
    assert pano_info, '瓦片全景信息加载失败'

    img_pano = pano_info.load_tiles(opt.no_background)
    print('pano shape:', img_pano.shape)
    if opt.grid_thickness:
        grid(img_pano, pano_info.tile_size(), colors=COLORS7, thickness=opt.grid_thickness)
    trace_image(img_pano)

    if opt.output:
        print('图片保存为：', opt.output)
        save_image(img_pano, opt.output)
        if opt.divide_size:
            size = size_parse(opt.divide_size)
            out_dir = Path(opt.output.stem)
            sparse_info = divide_sparse(img_pano, pano_info.tiles, out_dir, size.width, size.height)
            sparse_info.background = str(opt.output)
            out_file = out_dir.with_suffix('.json')
            print('稀疏全景信息信息保存到：', out_file)
            save_json(sparse_info, out_file)


if __name__ == '__main__':
    main()



#!/opt/ias/env/bin/python

import cv2
import numpy as np
from jvi.geo.point2d import Point
from jvi.geo.rectangle import Rect
from jvi.image.util import ndarray_rect, ndarray_size
from numpy import uint8


def find_template(src: np.ndarray, templ: np.ndarray) -> tuple[Point, float]:
    """多通道图像模板匹配，逐像素搜索，大图难行"""

    assert src.dtype == templ.dtype
    assert src.shape[2] == templ.shape[2]  # 图像通道数一致

    src = src.astype(int)
    templ = templ.astype(int)

    size = ndarray_size(templ)

    h = templ.shape[0]
    w = templ.shape[1]
    h_max = src.shape[0] - h + 1
    w_max = src.shape[1] - w + 1
    var_min = float('inf')
    pos = Point(0, 0)
    for y in range(h_max + 1):
        # if y % 100 == 0:
        print('搜索行：', y)
        for x in range(w_max + 1):
            print('搜索列：', x, var_min)
            r = Rect(x, y, w, h)
            roi = ndarray_rect(src, r)
            v = np.var(roi - templ)
            if v < var_min:
                var_min = v
                pos = Point(x, y)
    return pos, var_min


def find_template2(src: np.ndarray, templ: np.ndarray) -> tuple[Point, float]:
    """单通道图像模板匹配"""

    # src = cv2.cvtColor(src, cv2.COLOR_BGR2GRAY)
    # templ = cv2.cvtColor(templ, cv2.COLOR_BGR2GRAY)

    method = cv2.TM_SQDIFF
    result = cv2.matchTemplate(src, templ, method)

    index = np.argmin(result)
    row = index // result.shape[1]
    col = index % result.shape[1]

    return Point(col, row), result[row, col]


def find_template_tile(src: np.ndarray, templ: np.ndarray) -> tuple[Point, float]:
    """多通道图像模板匹配，基于对齐的块"""

    assert src.dtype == templ.dtype
    assert src.shape[2] == templ.shape[2]  # 图像通道数一致

    src = src.astype(int)
    templ = templ.astype(int)

    rect = Rect.from_size(ndarray_size(src))
    size = ndarray_size(templ)

    var_min = float('inf')
    pos = Point(0, 0)
    rects = rect.to_tiles(size=size, need_round=True)
    for r in rects:
        # print('搜索列：', r, var_min)
        roi = ndarray_rect(src, r)
        v = np.var(roi - templ)
        if v < var_min:
            var_min = v
            pos = r.lt()

    return pos, var_min


def find_template_test():
    img = np.ones([5, 6, 1], dtype=uint8)
    img[3, 3] = 10

    temp = np.ones([2, 2, 1], dtype=uint8)
    temp[1, 1] = 11

    print('img:', img)
    print('temp:', temp)

    p, v = find_template2(img, temp)
    print(p, v)


if __name__ == '__main__':
    find_template_test()



from typing import Iterable

from jvi.geo.point2d import Point, Points
from jvi.geo.rectangle import Rect
from jvi.geo.size2d import Size


def point_cs_trans_in_size(point: Point, src_cs: Size, dst_cs: Size) -> Point:
    """点变换坐标系-坐标系缩放, 点未归一化"""
    x = point.x * dst_cs.width / src_cs.width
    y = point.y * dst_cs.height / src_cs.height
    return Point(x=x, y=y)


def point_cs_trans_in_rect(point: Point, src_cs: Rect, dst_cs: Rect) -> Point:
    """点变换坐标系-区域到区域, 点未归一化"""
    return point_cs_trans_in_size(point - src_cs.lt(), src_cs.size(), dst_cs.size()) + dst_cs.lt()


def points_cs_trans_in_rect(points: Iterable[Point], src_cs: Rect, dst_cs: Rect) -> Points:
    """点变换坐标系-区域到区域, 点未归一化"""
    return [point_cs_trans_in_rect(p, src_cs, dst_cs) for p in points]


def rect_cs_trans_in_size(rect: Rect, src_cs: Size, dst_cs: Size) -> Rect:
    """矩形变换坐标系-区域到区域, 点未归一化"""
    x = rect.x * dst_cs.width / src_cs.width
    y = rect.y * dst_cs.height / src_cs.height
    w = rect.width * dst_cs.width / src_cs.width
    h = rect.height * dst_cs.height / src_cs.height
    return Rect(x, y, w, h)



#!/opt/ias/env/bin/python

import argparse
from pathlib import Path

import cv2
from jvi.geo.rectangle import Rect
from jvi.geo.size2d import size_parse
from jvi.image.image_nda import ImageNda

epilog = '''
Examples:

    jvi-resize src.png dst.png -c 256x256
    
'''


def main():
    parser = argparse.ArgumentParser(description='图片缩放工具', epilog=epilog)
    parser.add_argument('src', type=Path, help='来源图片')
    parser.add_argument('dst', type=Path, help='目的图片')
    parser.add_argument('-c', '--crop_size', type=str, help='目标图片裁切尺寸(中心裁切)')
    parser.add_argument('-s', '--scale', type=float, default=1, help='图片缩放比例')
    parser.add_argument('-i', '--interpolation', type=int, default=1, help='差之方法: 1-INTER_LINEAR, 2-INTER_CUBIC')

    parser.add_argument('-v', '--verbose', action='store_true', help='显示详细信息')
    opt = parser.parse_args()

    assert opt.interpolation in (1, 2)

    src = ImageNda.load(str(opt.src), cv2.IMREAD_UNCHANGED)  # BGR
    assert src is not None
    dst = cv2.resize(src, (0, 0), None, fx=opt.scale, fy=opt.scale, interpolation=opt.interpolation)

    dst_size = dst.size()

    if opt.crop_size:
        crop_size = size_parse(opt.crop_size)

        if dst_size.contains(crop_size):
            r = Rect.from_cs(dst_size.center(), crop_size)
            crop = dst.roi(r)
        else:
            crop = ImageNda(crop_size, src.shape[2])
            center = crop_size.center()
            r = Rect.from_cs(center, dst_size).round()
            roi = crop.roi(r)
            cv2.copyTo(dst, None, roi)
        dst = crop
        if opt.verbose:
            cv2.imshow('crop', crop)

    if opt.verbose:
        cv2.imshow('src', src)
        cv2.imshow('dst', dst)
        cv2.waitKey(0)

    cv2.imwrite(str(opt.dst), dst)
    cv2.destroyAllWindows()


if __name__ == '__main__':
    main()



#!/opt/ias/env/bin/python

import argparse
from pathlib import Path

import cv2
from jcx.sys.fs import files_in, real_exe_path

'''
图片超分辨率工具

不适合整合为resize工具，因为分辨率受限。

'''


def resize_file(model, src: Path, dst: Path, scale: int, verbose: bool):
    img_src = ImageNda.load(str(src))
    if img_src is None:
        print("[ERROR] load image fail: " + str(src))
        return
    print('upsample source image ...')
    img_dst = model.upsample(img_src)
    print('imwrite image to:', dst)
    cv2.imwrite(str(dst), img_dst)

    if verbose:
        img_linear = cv2.resize(img_src, None, fx=scale, fy=scale, interpolation=cv2.INTER_LINEAR)
        img_bicubic = cv2.resize(img_src, None, fx=scale, fy=scale, interpolation=cv2.INTER_CUBIC)

        cv2.imshow('img_src', img_src)
        cv2.imshow('img_dst', img_dst)
        cv2.imshow('img_linear', img_linear)
        cv2.imshow('img_bicubic', img_bicubic)

        cv2.waitKey(0)


def main():
    parser = argparse.ArgumentParser(description='图片超分辨率工具')
    parser.add_argument('src', type=Path, help='来源图片文件/目录')
    parser.add_argument('dst', type=Path, help='目的图片文件/目录')
    parser.add_argument('-s', '--scale', type=int, default=2, help='图片放大倍数: 2,3,4')
    parser.add_argument('-e', '--ext', type=str, help='目录内待转换文件扩展名')
    parser.add_argument('-a', '--algorithm', type=str, default='edsr', help='放缩算法')
    parser.add_argument('-v', '--verbose', action='store_true', help='显示详细信息')
    opt = parser.parse_args()

    model_dir = real_exe_path().parent.parent / 'model/superres'
    models = {2: 'EDSR_x2.pb', 3: 'EDSR_x3.pb', 4: 'EDSR_x4.pb'}

    if opt.scale not in models:
        print('[ERROR] 无效放大倍数')
        return -2

    model = cv2.dnn_superres.DnnSuperResImpl_create()
    model_file = model_dir / models[opt.scale]
    print('load model:', model_file)
    model.readModel(str(model_file))

    model.setModel(opt.algorithm, opt.scale)

    if opt.ext:
        files = files_in(opt.src, opt.ext)
        for src in files:
            dst = opt.dst / src.name
            resize_file(model, src, dst, opt.scale, opt.verbose)
    else:
        resize_file(model, opt.src, opt.dst, opt.scale, opt.verbose)

    cv2.destroyAllWindows()


if __name__ == '__main__':
    main()



from abc import abstractmethod
from enum import IntEnum
from typing import TypeAlias, TypeVar, Protocol

from jcx.m.number import Real
from jcx.sys.fs import StrPath
from jvi.drawing.color import Color
from jvi.geo.rectangle import Rect
from jvi.geo.size2d import Size

Self = TypeVar('Self', bound='Image')

Shape2D: TypeAlias = tuple[int, int]
"""三维形状"""
Shape3D: TypeAlias = tuple[int, int, int]
"""三维形状"""

ImageShape: TypeAlias = tuple


# ImageShape: TypeAlias = Shape3D | Shape2D


class DType(IntEnum):
    """数据类型"""
    U8 = 1
    U16 = 2
    U32 = 4
    U64 = 8
    I8 = 11
    I16 = 12
    I32 = 14
    I64 = 18
    F8 = 21
    F16 = 22
    F32 = 24
    F64 = 28


ImageColor: TypeAlias = Real | tuple | Color
"""Image可用颜色类型"""


class Image(Protocol):
    """图像抽象类"""

    def __str__(self) -> str:
        """图像描述信息"""
        s = self.size()
        return '%s(%dx%dx%d,%s)' % (self.__class__.__name__, s.width, s.height, self.channel_num(), self.dtype())

    @abstractmethod
    def __bool__(self) -> bool:
        """判断图像是否有效"""
        pass

    @abstractmethod
    def size(self) -> Size:
        """获取图像尺寸"""
        pass

    def same_size(self, other: Self) -> bool:
        """判定尺寸相等"""
        return self.size() == other.size()

    @abstractmethod
    def channel_num(self) -> int:
        """获取图像通道数"""
        pass

    def same_channel_num(self, other: Self) -> bool:
        """判定通道数相等"""
        return self.channel_num() == other.channel_num()

    @abstractmethod
    def channel_at(self: Self, idx: int) -> Self:
        """获取图像指定通道"""
        pass

    @abstractmethod
    def channels(self) -> list[Self]:
        """获取图像所有通道"""
        pass

    @abstractmethod
    def shape(self) -> Shape3D:
        """获取图像形状: (高, 宽, 通道)"""
        s = self.size()
        return s.to_shape3d_i(self.channel_num())

    def same_shape(self, other: Self) -> bool:
        """判定形状相等"""
        return self.shape() == other.shape()

    @abstractmethod
    def dtype(self) -> DType:
        """获取数据类型"""
        pass

    def same_dtype(self, other: Self) -> bool:
        """判定数据类型相等"""
        return self.dtype() == other.dtype()

    def same_shape_type(self, other: Self) -> bool:
        """判定形状&类型相等"""
        return self.same_shape(other) and self.same_dtype(other)

    @abstractmethod
    def clone(self: Self) -> Self:
        """克隆图片对象"""
        pass

    @abstractmethod
    def copy_to(self, other: Self) -> None:
        """复制数据到目标图像"""
        pass

    @abstractmethod
    def convert_to(self, other: Self) -> None:
        """复制转换数据类型到目标图像"""
        pass

    @abstractmethod
    def set_to(self, color: ImageColor) -> None:
        """图像设置为指定颜色"""
        pass

    @abstractmethod
    def roi(self: Self, r: Rect) -> Self:
        """获取ndarray区域"""
        pass

    @abstractmethod
    def save(self, path: StrPath) -> bool:
        """保存图片"""
        pass



from dataclasses import dataclass
from pathlib import Path
from typing import TypeAlias, Protocol

from jcx.sys.fs import StrPath
from jvi.geo.point2d import Point
from jvi.geo.size2d import Size, SIZE_HD
from jvi.gui.image_viewer import ImageViewer
from jvi.image.image_nda import ImageNda
from rustshed import Option, Null


@dataclass
class ImageEntry(Protocol):
    """图片信息"""

    def load_image(self) -> ImageNda:
        """加载图片"""
        pass

    def image_file(self) -> Path:
        """获取图片路径"""
        pass

    def draw_on(self, canvas: ImageNda, pos: Point) -> None:
        """把图片信息绘制在画板上"""
        pass


ImageEntries: TypeAlias = list[ImageEntry]
"""图片信息集合"""


class RecordViewer(ImageViewer):
    """记录图片查看窗口"""

    def __init__(self, title: str, size: Size = SIZE_HD):
        super().__init__(title, size)
        self._records: ImageEntries = []

    def on_draw(self, canvas: ImageNda, pos: Point) -> None:
        """设置窗口重绘事件响应"""
        self.record().draw_on(canvas, pos)

    def image_at(self, index: int) -> ImageNda:
        """获取指定索引的图片"""
        return self._records[index].load_image()

    def cur_image_file(self) -> Path:
        return self._records[self._index].image_file()

    def image_count(self) -> int:
        """获取图片总数"""
        return len(self._records)

    def set_records(self, records: ImageEntries) -> None:
        """设置记录"""
        # self._records = sorted(records)
        self._records = records
        if self._records:
            self.change_background()

    def record(self) -> ImageEntry:
        """获取当前记录"""
        return self._records[self._index]


@dataclass
class FileRecord(ImageEntry):
    """文件记录"""
    path: Path
    """图片路径"""
    image: Option[ImageNda] = Null
    """图片对象, 内存中的图片数据"""

    def load_image(self) -> ImageNda:
        """加载图片"""
        return ImageNda.load(self.path)

    def image_file(self) -> Path:
        """获取图片路径"""
        return self.path

    def draw_on(self, canvas: ImageNda, _pos: Point) -> None:
        """把记录绘制在画板上"""
        # label = self.path.name
        # draw_boxf(canvas, Rect(0.25, 0.25, 0.5, 0.5), GREEN, label, 3)
        pass


FileRecords: TypeAlias = list[FileRecord]
"""文件记录列表"""


def load_dir_records(src_dir: StrPath) -> FileRecords:
    """加载目录下的图片信息记录"""
    src_dir = Path(src_dir)
    # print(src_dir)
    files = sorted(src_dir.rglob('*.jpg'))

    rs = [FileRecord(f) for f in files]
    return rs


def main():
    folder = '/home/jiang/ws/trash/dates/2023-02-25/image'

    rs = load_dir_records(folder)
    assert len(rs) > 0

    win = RecordViewer('files://' + folder)
    win.set_records(rs)

    print('image number:', win.image_count())
    win.run()


if __name__ == '__main__':
    main()



#!/opt/ias/env/bin/python

import argparse
from pathlib import Path
from typing import Optional

from jcx.text.txt_json import save_json
from jvi.image.tile import panorama_from_dir


def main():
    parser = argparse.ArgumentParser(description='目录图片瓦片平铺工具')
    parser.add_argument('src_dir', type=Path, help='来源图片目录')
    parser.add_argument('-p', '--panorama', type=Optional[Path], default=None, help='匹配全景文件')
    parser.add_argument('-e', '--ext', type=str, default='.png', help='来源图片文件扩展名')
    parser.add_argument('-c', '--cols', required=True, type=int, help='瓦片列数')
    parser.add_argument('-r', '--rows', type=int, help='瓦片行数')
    parser.add_argument('-v', '--verbose', action='store_true', help='显示详细信息')
    opt = parser.parse_args()

    panorama = opt.panorama or opt.src_dir.with_suffix('.json')
    print('匹配信息输出到：', panorama)

    info = panorama_from_dir(opt.src_dir, cols=opt.cols, rows=opt.rows, ext=opt.ext, verbose=opt.verbose)
    save_json(info, panorama)


if __name__ == '__main__':
    main()



#!/opt/ias/env/bin/python

import argparse
from pathlib import Path

from jcx.sys.fs import files_in
from jvi.drawing.color import YOLO_GRAY
from jvi.geo.rectangle import Rect
from jvi.geo.size2d import size_parse
from jvi.image.image_nda import ImageNda
from jvi.image.trace import trace_image

epilog = '''
Examples:

    jvi-letterbox src_dir dst_dir -S 256x256
    
'''


def main() -> None:
    parser = argparse.ArgumentParser(description='图片嵌入工具', epilog=epilog)
    parser.add_argument('src_dir', type=Path, help='来源图片目录')
    parser.add_argument('dst_dir', type=Path, help='目的图片目录')
    parser.add_argument('-S', '--dst_size', type=str, default='HD', help='目标图片裁切尺寸(中心裁切)')
    parser.add_argument('-v', '--verbose', action='store_true', help='显示详细信息')
    opt = parser.parse_args()

    dst_size = size_parse(opt.dst_size)
    assert dst_size

    files = files_in(opt.src_dir, '.jpg')
    assert len(files)
    for src_file in files:
        src = ImageNda.load(src_file)
        assert src is not None
        dst = ImageNda(dst_size, color=YOLO_GRAY)
        center = dst_size.center()
        r = Rect.from_cs(center, src.size()).round()
        src.copy_to(dst.roi(r))
        dst_file = Path(opt.dst_dir, src_file.name)
        trace_image(dst, dst_file.name)
        dst.save(dst_file)


if __name__ == '__main__':
    main()



from typing import Optional

import cv2
import numpy as np
from jvi.drawing.color import Color
from jvi.geo.point2d import Points, Point
from jvi.geo.rectangle import Rect
from jvi.geo.size2d import absolutize_points, Size
from jvi.image.image_nda import ImageNda, new_gray, ImageNdas


def np_shape(size: Size) -> tuple[int, int]:
    """Size转化成numpy形状"""
    return int(size.height), int(size.width)


def points_to_mat(roi: Points) -> np.ndarray:
    """点集转换为mat"""
    arr = [[p.x, p.y] for p in roi]
    # print('arr', arr)
    return np.array([arr], dtype=np.int32)


def mat_to_points(mat: np.ndarray) -> Points:
    """mat转换为点集"""
    mat = mat.reshape(-1, 2)
    return [Point(p[0], p[1]) for p in mat]


def make_mask(size: Size, roi: Points, mask: Optional[ImageNda] = None) -> ImageNda:
    """制造指定ROI掩码图"""
    shape = (size.height, size.width)
    if isinstance(mask, ImageNda) and mask.shape() == shape:  # 已经存在尺寸合适mark,则返回
        return mask

    mask = new_gray(size)

    roi = absolutize_points(roi, size)
    roi_mat = points_to_mat(roi)

    cv2.fillPoly(mask.data(), roi_mat, 255)
    return mask


def make_roi_surround_color(img: ImageNda, roi: Points, channel: int = 0) -> None:
    """让图像指定ROI之外部分变成单通道颜色, 用以凸显ROI. channel: 0-蓝色, 1-绿色, 2-红色"""
    mask = make_mask(img.size(), roi)
    cv2.bitwise_not(mask.data(), mask.data())
    # cv2.imshow("mask", mask)

    gray = cv2.cvtColor(img.data(), cv2.COLOR_RGB2GRAY)
    blue = ImageNda(img.size())
    blue.data()[:, :, channel] = gray

    copy_mask(blue, mask, img)


def copy_mask(src: ImageNda, mask: ImageNda, dst: Optional[ImageNda] = None,
              bg_color: Optional[Color] = None) -> ImageNda:
    """根据mask复制"""
    if dst is None or not src.same_shape_type(dst):
        dst = ImageNda.new_as(src, color=bg_color)
    data = cv2.copyTo(src.data(), mask.data(), dst.data())
    return ImageNda(data=data)


def image_to_tiles(image: ImageNda, size: Size) -> ImageNdas:
    """获取ndarray瓦片图"""
    rs = Rect.from_size(image.size()).to_tiles(size=size)
    return [image.roi(r) for r in rs]



import inspect
import sys
from random import randint
from typing import Final, Optional, Self

from pydantic import BaseModel


class Color(BaseModel):
    """RGB颜色"""

    @classmethod
    def try_parse(cls, name: str) -> Optional[Self]:
        """从颜色名称或者#RGB建立颜色"""
        if name.startswith('#'):
            s = name.replace('#', '0x')
            i = int(s, 16)
            b = i % 256
            i = i // 256
            g = i % 256
            r = i // 256
            return Color.new(r, g, b)
        else:
            name = name.upper().replace(' ', '_')
            return colors.get(name, None)

    @classmethod
    def parse(cls, name: str) -> Self:
        """从颜色名称或者#RGB建立颜色"""
        c = cls.try_parse(name)
        assert c, f'Invalid color: {name}'
        return c

    @classmethod
    def new(cls, r: int, g: int, b: int) -> Self:
        """从RGB建立颜色"""
        return Color(r=r, g=g, b=b)

    r: int = 0
    g: int = 0
    b: int = 0

    def rgb(self) -> tuple[int, int, int]:
        """获取RGB"""
        return self.r, self.g, self.b

    def bgr(self) -> tuple[int, int, int]:
        """获取BGR"""
        return self.b, self.g, self.r

    def inverse(self) -> Self:
        """获取反色"""
        return Color(r=255 - self.r, g=255 - self.g, b=255 - self.b)

    def as_int(self) -> int:
        """颜色转换为单个整数"""
        return self.r << 16 | self.g << 8 | self.b

    def __str__(self) -> str:
        return "#%02X%02X%02X" % (self.r, self.g, self.b)

    def __len__(self) -> int:
        return 3


type Colors = list[Color]
"""颜色组"""

# 颜色定义

PINK: Final = Color.new(255, 192, 203)
"""粉红"""
CRIMSON: Final = Color.new(220, 20, 60)
"""猩红"""
LAVENDER_BLUSH: Final = Color.new(255, 240, 245)
"""淡紫红"""
PALE_VIOLET_RED: Final = Color.new(219, 112, 147)
"""苍白的紫罗兰红色"""
HOT_PINK: Final = Color.new(255, 105, 180)
"""热粉红"""
DEEP_PINK: Final = Color.new(255, 20, 147)
"""深粉红"""
MEDIUM_VIOLET_RED: Final = Color.new(199, 21, 133)
"""适中的紫罗兰红色"""
ORCHID: Final = Color.new(218, 112, 214)
"""兰花紫"""
THISTLE: Final = Color.new(216, 191, 216)
"""蓟"""
PLUM: Final = Color.new(221, 160, 221)
"""李子色"""
VIOLET: Final = Color.new(238, 130, 238)
"""紫罗兰"""
MAGENTA: Final = Color.new(255, 0, 255)
"""洋红"""
FUCHSIA: Final = Color.new(255, 0, 255)
"""紫红色"""
DARK_MAGENTA: Final = Color.new(139, 0, 139)
"""深洋红色"""
PURPLE: Final = Color.new(128, 0, 128)
"""紫色"""
MEDIUM_ORCHID: Final = Color.new(186, 85, 211)
"""适中的兰花紫"""
DARK_VIOLET: Final = Color.new(148, 0, 211)
"""深紫罗兰色"""
DARK_ORCHID: Final = Color.new(153, 50, 204)
"""深兰花紫"""
INDIGO: Final = Color.new(75, 0, 130)
"""靛青"""
BLUE_VIOLET: Final = Color.new(138, 43, 226)
"""紫罗兰的蓝色"""
MEDIUM_PURPLE: Final = Color.new(147, 112, 219)
"""适中的紫色"""
MEDIUM_SLATE_BLUE: Final = Color.new(123, 104, 238)
"""适中的板岩暗蓝灰色"""
SLATE_BLUE: Final = Color.new(106, 90, 205)
"""板岩暗蓝灰色"""
DARK_SLATE_BLUE: Final = Color.new(72, 61, 139)
"""深板岩暗蓝灰色"""
LAVENDER: Final = Color.new(230, 230, 250)
"""薰衣草花的淡紫色"""
GHOST_WHITE: Final = Color.new(248, 248, 255)
"""幽灵的白色"""
BLUE: Final = Color.new(0, 0, 255)
"""蓝"""
MEDIUM_BLUE: Final = Color.new(0, 0, 205)
"""适中的蓝色"""
MIDNIGHT_BLUE: Final = Color.new(25, 25, 112)
"""午夜的蓝色"""
DARK_BLUE: Final = Color.new(0, 0, 139)
"""深蓝色"""
NAVY: Final = Color.new(0, 0, 128)
"""海军蓝"""
ROYAL_BLUE: Final = Color.new(65, 105, 255)
"""皇家蓝"""
CORNFLOWER_BLUE: Final = Color.new(100, 149, 237)
"""矢车菊的蓝色"""
LIGHT_STEEL_BLUE: Final = Color.new(176, 196, 222)
"""淡钢蓝"""
LIGHT_SLATE_GRAY: Final = Color.new(119, 136, 153)
"""浅石板灰"""
SLATE_GRAY: Final = Color.new(112, 128, 144)
"""石板灰"""
DODGER_BLUE: Final = Color.new(30, 144, 255)
"""道奇蓝"""
ALICE_BLUE: Final = Color.new(240, 248, 255)
"""爱丽丝蓝"""
STEEL_BLUE: Final = Color.new(70, 130, 180)
"""钢蓝"""
LIGHT_SKY_BLUE: Final = Color.new(135, 206, 250)
"""淡天蓝色"""
SKY_BLUE: Final = Color.new(135, 206, 235)
"""天蓝色"""
DEEP_SKY_BLUE: Final = Color.new(0, 191, 255)
"""深天蓝"""
LIGHT_BLUE: Final = Color.new(173, 216, 230)
"""淡蓝色"""
POWDER_BLUE: Final = Color.new(176, 224, 230)
"""浅灰蓝"""
CADET_BLUE: Final = Color.new(95, 158, 160)
"""军校蓝"""
AZURE: Final = Color.new(240, 255, 255)
"""蔚蓝色"""
LIGHT_CYAN: Final = Color.new(224, 255, 255)
"""浅青色"""
PALE_TURQUOISE: Final = Color.new(175, 238, 238)
"""苍白的绿宝石"""
CYAN: Final = Color.new(0, 255, 255)
"""青色"""
AQUA: Final = Color.new(0, 255, 255)
"""水绿色"""
DARK_TURQUOISE: Final = Color.new(0, 206, 209)
"""深绿宝石"""
DARK_SLATE_GRAY: Final = Color.new(47, 79, 79)
"""深石板灰"""
DARK_CYAN: Final = Color.new(0, 139, 139)
"""深青色"""
TEAL: Final = Color.new(0, 128, 128)
"""水鸭色"""
MEDIUM_TURQUOISE: Final = Color.new(72, 209, 204)
"""适中的绿宝石"""
LIGHT_SEA_GREEN: Final = Color.new(32, 178, 170)
"""浅海洋绿"""
TURQUOISE: Final = Color.new(64, 224, 208)
"""绿宝石"""
AQUAMARINE: Final = Color.new(127, 255, 212)
"""绿玉，碧绿色"""
MEDIUM_AQUAMARINE: Final = Color.new(102, 205, 170)
"""适中的碧绿色"""
MEDIUM_SPRING_GREEN: Final = Color.new(0, 250, 154)
"""适中的春天的绿色"""
MINT_CREAM: Final = Color.new(245, 255, 250)
"""薄荷奶油"""
SPRING_GREEN: Final = Color.new(0, 255, 127)
"""春天的绿色"""
MEDIUM_SEA_GREEN: Final = Color.new(60, 179, 113)
"""适中的海洋绿"""
SEA_GREEN: Final = Color.new(46, 139, 87)
"""海洋绿"""
HONEYDEW: Final = Color.new(240, 255, 240)
"""蜂蜜"""
LIGHT_GREEN: Final = Color.new(144, 238, 144)
"""淡绿色"""
PALE_GREEN: Final = Color.new(152, 251, 152)
"""苍白的绿色"""
DARK_SEA_GREEN: Final = Color.new(143, 188, 143)
"""深海洋绿"""
LIME_GREEN: Final = Color.new(50, 205, 50)
"""酸橙绿"""
LIME: Final = Color.new(0, 255, 0)
"""酸橙色"""
FOREST_GREEN: Final = Color.new(34, 139, 34)
"""森林绿"""
GREEN: Final = Color.new(0, 128, 0)
"""绿"""
DARK_GREEN: Final = Color.new(1, 100, 0)
"""深绿色"""
CHARTREUSE: Final = Color.new(127, 255, 0)
"""查特酒绿，淡黄绿色"""
LAWN_GREEN: Final = Color.new(124, 252, 0)
"""草坪绿"""
GREEN_YELLOW: Final = Color.new(173, 255, 47)
"""绿黄色"""
DARK_OLIVE_GREEN: Final = Color.new(85, 107, 47)
"""深橄榄绿"""
YELLOW_GREEN: Final = Color.new(154, 205, 50)
"""黄绿"""
OLIVE_DRAB: Final = Color.new(107, 142, 35)
"""橄榄土褐色"""
BEIGE: Final = Color.new(245, 245, 220)
"""米色，浅褐色"""
LIGHT_GOLDENROD_YELLOW: Final = Color.new(250, 250, 210)
"""浅秋麒麟黄"""
IVORY: Final = Color.new(255, 255, 240)
"""象牙"""
LIGHT_YELLOW: Final = Color.new(255, 255, 224)
"""浅黄色"""
YELLOW: Final = Color.new(255, 255, 0)
"""黄"""
OLIVE: Final = Color.new(128, 128, 0)
"""橄榄色"""
DARK_KHAKI: Final = Color.new(189, 183, 107)
"""深卡其布"""
LEMON_CHIFFON: Final = Color.new(255, 250, 205)
"""柠檬薄纱"""
PALE_GOLDENROD: Final = Color.new(238, 232, 170)
"""灰秋麒麟"""
KHAKI: Final = Color.new(240, 230, 140)
"""卡其布"""
GOLD: Final = Color.new(255, 215, 0)
"""金色"""
CORNSILK: Final = Color.new(255, 248, 220)
"""玉米色"""
GOLDENROD: Final = Color.new(218, 165, 32)
"""秋麒麟色"""
DARK_GOLDENROD: Final = Color.new(184, 134, 11)
"""深秋麒麟"""
FLORAL_WHITE: Final = Color.new(255, 250, 240)
"""花卉白"""
OLDLACE: Final = Color.new(253, 245, 230)
"""老饰带"""
WHEAT: Final = Color.new(245, 222, 179)
"""小麦色"""
MOCCASIN: Final = Color.new(255, 228, 181)
"""鹿皮鞋"""
ORANGE: Final = Color.new(255, 165, 0)
"""橙色"""
PAPAYA_WHIP: Final = Color.new(255, 239, 213)
"""番木瓜色"""
BLANCHED_ALMOND: Final = Color.new(255, 235, 205)
"""漂白的杏仁"""
NAVAJO_WHITE: Final = Color.new(255, 222, 173)
"""纳瓦霍白"""
ANTIQUE_WHITE: Final = Color.new(250, 235, 215)
"""古典白"""
TAN: Final = Color.new(210, 180, 140)
"""棕褐色，茶色"""
BURLYWOOD: Final = Color.new(222, 184, 135)
"""原木色"""
BISQUE: Final = Color.new(255, 228, 196)
"""浓汤色"""
DARK_ORANGE: Final = Color.new(255, 140, 0)
"""深橙黄"""
LINEN: Final = Color.new(250, 240, 230)
"""亚麻布色"""
PERU: Final = Color.new(205, 133, 63)
"""秘鲁色"""
PEACH_PUFF: Final = Color.new(255, 218, 185)
"""桃色"""
SANDY_BROWN: Final = Color.new(244, 164, 96)
"""沙棕色"""
CHOCOLATE: Final = Color.new(210, 105, 30)
"""巧克力色"""
SADDLE_BROWN: Final = Color.new(139, 69, 19)
"""马鞍棕色"""
SEA_SHELL: Final = Color.new(255, 245, 238)
"""海贝色"""
SIENNA: Final = Color.new(160, 82, 45)
"""赭色"""
LIGHT_SALMON: Final = Color.new(255, 160, 122)
"""浅肉色"""
CORAL: Final = Color.new(255, 127, 80)
"""珊瑚色"""
ORANGE_RED: Final = Color.new(255, 69, 0)
"""橙红色"""
DARK_SALMON: Final = Color.new(233, 150, 122)
"""深肉色"""
TOMATO: Final = Color.new(255, 99, 71)
"""番茄色"""
MISTY_ROSE: Final = Color.new(255, 228, 225)
"""薄雾玫瑰色"""
SALMON: Final = Color.new(250, 128, 114)
"""鲜肉色"""
SNOW: Final = Color.new(255, 250, 250)
"""雪白"""
LIGHT_CORAL: Final = Color.new(240, 128, 128)
"""浅珊瑚色"""
ROSY_BROWN: Final = Color.new(188, 143, 143)
"""玫瑰棕色"""
INDIAN_RED: Final = Color.new(205, 92, 92)
"""印度红"""
RED: Final = Color.new(255, 0, 0)
"""红"""
BROWN: Final = Color.new(165, 42, 42)
"""褐色"""
FIRE_BRICK: Final = Color.new(178, 34, 34)
"""耐火砖色"""
DARK_RED: Final = Color.new(139, 0, 0)
"""深红"""
MAROON: Final = Color.new(128, 0, 0)
"""栗色"""
WHITE: Final = Color.new(255, 255, 255)
"""白"""
WHITE_SMOKE: Final = Color.new(245, 245, 245)
"""白烟"""
GAINSBORO: Final = Color.new(220, 220, 220)
"""淡灰色"""
LIGHT_GREY: Final = Color.new(211, 211, 211)
"""浅灰"""
SILVER: Final = Color.new(192, 192, 192)
"""银白"""
DARK_GRAY: Final = Color.new(169, 169, 169)
"""深灰"""
GRAY: Final = Color.new(128, 128, 128)
"""灰色"""
DIM_GRAY: Final = Color.new(105, 105, 105)
"""暗灰色"""
BLACK: Final = Color.new(0, 0, 0)
"""黑"""
YOLO_GRAY: Final = Color.new(114, 114, 114)  # /opt/ias/env/lib/yolo5/utils/general.py
"""YOLO灰色"""

COLORS7: Final = [RED, ORANGE, YELLOW, GREEN, CYAN, BLUE, PURPLE]
"""七色"""

W3C16: Final = [BLACK, GREEN, SILVER, LIME, GRAY, OLIVE, WHITE, YELLOW,
                MAROON, NAVY, RED, BLUE, PURPLE, TEAL, FUCHSIA, AQUA]
"""W3C16色"""


def find_mod_colors() -> dict[str, Color]:
    """搜索模块内所有预定义颜色"""
    ns = inspect.getmembers(sys.modules[__name__])
    cs = {}
    for k, v in ns:
        if isinstance(v, Color):
            cs[k] = v
    return cs


colors = find_mod_colors()
"""预定义颜色字典"""


def random_color() -> Color:
    """获取随机颜色"""
    return Color.new(randint(0, 256), randint(0, 256), randint(0, 256))



from abc import abstractmethod
from dataclasses import dataclass
from typing import Optional

from jcx.ui.key import Key
from jvi.geo.size2d import Size, SIZE_HD
from jvi.gui.image_win import ImageWin
from jvi.image.image_nda import ImageNda


@dataclass
class ImageRecord:
    """传感器参数"""
    file: str = ''
    image: Optional[ImageNda] = None


class ImageViewer(ImageWin):
    """图像查看窗口"""

    def __init__(self, title: str, size: Size = SIZE_HD):
        super().__init__(title, size)
        self.key_tab = {
            ord("a"): -1,
            ord("d"): 1,
            ord("w"): 10,
            ord("s"): -10
        }
        self._src_dir = None  # 派生类使用
        self._index = 0
        self._pre_index = -1
        self._preload_step = 1
        self._preload_index = 0
        self.help_msgs = [
            "导航按键表：",
            "  [a] 导航到前一副图片",
            "  [d] 导航到后一副图片",
            "  [w] 导航到前十副图片",
            "  [s] 导航到后十副图片",
        ]

    @abstractmethod
    def image_at(self, index: int) -> ImageNda:
        """获取指定索引的图片"""
        pass

    @abstractmethod
    def image_count(self) -> int:
        """获取图片总数"""
        pass

    def on_change_image(self, index: int) -> None:
        """当图片改变时更新数据"""
        pass

    def on_key(self, key: int) -> int:
        """按键处理程序"""
        if key == Key.ESC:
            return 1
        elif key == -1:
            return 0
        elif key == Key.F1:
            self.show_help()
        else:
            # print('key:', key)
            self.jump_to(self.key_tab.get(key, 0))
        return 0

    def jump_to(self, offset: int) -> None:
        """导航到指定偏移量"""
        self._index += offset
        self._index = max(self._index, 0)
        self._index = min(self._index, self.image_count() - 1)
        self.change_background()

        self._preload_step = offset
        self._preload_index = self._index

    def change_background(self) -> None:
        """改变背景"""
        if self._index != self._pre_index:
            image = self.image_at(self._index)
            self.set_background(image)
            self._pre_index = self._index
            self.on_change_image(self._index)

    def show_help(self) -> None:
        """显示当前帮助"""
        for s in self.help_msgs:
            print(s)

    def on_idle(self) -> None:
        self._preload_index += self._preload_step
        if 0 <= self._preload_index < self.image_count():
            self.image_at(self._preload_index)



#!/opt/ias/env/bin/python

import argparse
from pathlib import Path

import cv2  # type: ignore
from jcx.sys.fs import files_in, stem_append
from jvi.geo.rectangle import Rect
from jvi.geo.size2d import Size
from jvi.image.image_nda import ImageNda

epilog = '''
Examples:

    jvi-cut src.png dst.png -a 1:1
    
'''


def cut_square(src_dir: Path, dst_dir: Path, ext: str):
    files = files_in(src_dir, ext)

    for src_file in files:
        src_im: ImageNda = ImageNda.load(src_file)
        src_size = src_im.size()
        side = min(src_size.width, src_size.height)
        dst_size = Size(side, side)
        tiles = Rect.from_size(src_size).to_tiles(size=dst_size)
        for i, r in enumerate(tiles):
            dst_im: ImageNda = src_im.roi(r)
            dst_file = dst_dir / src_file.name
            dst_file = stem_append(dst_file, f'_{i}')
            print(f'save to: {dst_file}')
            dst_im.save(dst_file)


def main() -> None:
    parser = argparse.ArgumentParser(description='图片切分工具', epilog=epilog)
    parser.add_argument('src_dir', type=Path, help='来源图片')
    parser.add_argument('dst_dir', type=Path, help='目的图片')
    parser.add_argument('-e', '--exp', type=str, default='.jpg', help='图片文件扩展名')
    parser.add_argument('-a', '--aspect', type=float, default=1, help='纵横比')
    parser.add_argument('-v', '--verbose', action='store_true', help='显示详细信息')
    opt = parser.parse_args()

    cut_square(opt.src_dir, opt.dst_dir, opt.exp)


if __name__ == '__main__':
    main()



#!/opt/ias/env/bin/python

import argparse
from pathlib import Path

import cv2
from jvi.geo.size2d import size_parse


def main():
    parser = argparse.ArgumentParser(description='图片转视频工具')
    parser.add_argument('src_dir', metavar='SRC_DIR', type=Path, help='来源图片所在目录')
    parser.add_argument('dst_file', metavar='DST_FILE', type=Path, help='目的视频文件')
    parser.add_argument('-p', '--pattern', type=str, default='*.jpg', help='图片文件模式(*.jpg)')
    parser.add_argument('-s', '--size', type=str, default='HD', help='输出视频尺寸(HD)')
    parser.add_argument('-f', '--fps', type=float, default=5, help='视频帧率(5)')
    parser.add_argument('-r', '--repeat', type=int, default=1, help='视频帧的重复次数(1)')
    parser.add_argument('-v', '--verbose', action='store_true', help='显示详细信息')
    opt = parser.parse_args()

    size = size_parse(opt.size)

    fourcc_tab = {'.mp4': 'mp4v', '.mjpg': 'mjpg', '.webp': 'webp'}  # require libwebp-dev
    # TODO: avc1/MJPG报错，https://stackoverflow.com/questions/52932157/ \
    #  opencv-ffmpeg-tag-0x34363268-h264-is-not-supported-with-codec/56723380

    fourcc = fourcc_tab.get(opt.dst_file.suffix)
    if not fourcc:
        print('[ERROR] 不支持的文件格式:', opt.dst_file.suffix)
        return -1
    fourcc = cv2.VideoWriter_fourcc(*fourcc)
    writer = cv2.VideoWriter(str(opt.dst_file), fourcc, opt.fps, size.to_tuple_int())

    files = opt.src_dir.rglob(opt.pattern)
    files = sorted(files)
    print('源图片目录：%s' % opt.src_dir)

    dst_img = None
    ok = 0
    err = 0
    for i, f in enumerate(files):
        src_img = ImageNda.load(str(f))  # BGR
        name = f.name
        if src_img is None:
            err += 1
            print(i, name, '\tERROR')
            continue
        else:
            ok += 1
            if opt.verbose:
                print(i, name, '\tOK')

        dst_img = cv2.resize(src_img, size.to_tuple_int(), dst_img)
        for _i in range(opt.repeat):
            writer.write(dst_img)

    print('\n压缩帧数：(%d/%d)\n' % (ok, ok + err))


if __name__ == '__main__':
    main()



#!/opt/ias/env/bin/python

import argparse
from pathlib import Path

import arrow
import cv2
from arrow import Arrow
from numpy import ndarray

from jcx.sys.fs import time_to_file, make_parents
from jcx.ui.key import Key
from jvi.geo.point2d import Point
from jvi.geo.size2d import size_parse, Size
from jvi.gui.video_viewer import VideoViewer

TITLE = 'video2image'


class ImageCapture(VideoViewer):
    """图片抓取"""

    def __init__(self, url, dst_dir: Path, size: Size = Size(), start_time: Arrow = arrow.now()):
        super().__init__(url, TITLE, size)
        self.__dst_dir = dst_dir
        self.__start_time = start_time

    def on_key(self, key: int):
        """按键响应"""
        if key == Key.BLANK:
            self.__save_image()
        else:
            return super().on_key(key)
        return 0

    def on_draw(self, canvas: ndarray, pos: Point):
        """设置窗口重绘事件响应"""
        # self.record().draw_on(canvas, pos)
        pass

    def __save_image(self):
        """保存图片"""
        image = self.image_at(self._index)
        if image is None:
            print('[ERROR] image is none')
            return

        interval = self._index * 1000000 / 25
        time = self.__start_time.shift(microseconds=interval)
        file = self.__dst_dir / time_to_file(time, '.jpg')
        make_parents(file)
        cv2.imwrite(str(file), image)
        print('save to:', file)


# file = 'file:///home/jiang/ws/lift/2021-08-19/1.mp4'

def main():
    parser = argparse.ArgumentParser(description='视频转图片工具')
    parser.add_argument('src_url', type=Path, help='来源视频URL')
    parser.add_argument('dst_dir', type=Path, help='目的图片所在目录')
    parser.add_argument('-e', '--ext', type=str, default='.jpg', help='图片文件扩展名')
    parser.add_argument('-s', '--size', type=str, default='HD', help='输出视频尺寸(HD)')
    parser.add_argument('-i', '--interval', type=float, default=1, help='时间间隔')
    parser.add_argument('-t', '--start_time', type=str, help='图片开始时间')
    parser.add_argument('-v', '--verbose', action='store_true', help='显示详细信息')
    opt = parser.parse_args()

    size = size_parse(opt.size)
    # TODO: start_time解析
    win = ImageCapture(opt.src_url, opt.dst_dir, size=Size(1280, 720))
    if not win.is_opened():
        print('[ERROR] cannot open url:', opt.src_url)
    print("frame count:", win.image_count())
    print('video size:', win.image_size())
    # print(opt.start_time)

    win.run()


if __name__ == '__main__':
    main()



import math
from copy import copy
from typing import Sequence, Iterable, Protocol, Self

import cv2
import numpy as np
from jcx.m import number
from jcx.m.number import Real1_2D, real_2d
from pydantic import BaseModel

from jvi.geo import is_normalized
from jvi.geo.protocol import PSize2D


class Point(BaseModel):
    """二维点"""
    x: float = 0
    y: float = 0

    @classmethod
    def zero(cls) -> Self:
        return Point()

    @classmethod
    def one(cls) -> Self:
        return Point(x=1, y=1)

    def __bool__(self) -> bool:
        """判定是否存在非零属性"""
        return bool(self.x or self.y)

    def clone(self) -> Self:
        """克隆对象"""
        return copy(self)

    def round_me(self) -> None:
        """近似成整数"""
        self.x = round(self.x)
        self.y = round(self.y)

    def round(self) -> Self:
        """近似成整数"""
        p = self.clone()
        p.round_me()
        return p

    def align_round_me(self, align: int) -> None:
        """近似对齐"""
        self.x = number.align_round(self.x, align)
        self.y = number.align_round(self.y, align)

    def align_round(self, align: int) -> Self:
        """近似对齐"""
        p = self.clone()
        p.align_round_me(align)
        return p

    def scale_me(self, n: Real1_2D) -> None:
        """缩放指定的倍数"""
        x, y = real_2d(n)
        self.x *= x
        self.y *= y

    def scale(self, n: Real1_2D) -> Self:
        """获取缩放指定的倍数的新对象"""
        p = self.clone()
        p.scale_me(n)
        return p

    def normalize_me(self, size: PSize2D) -> None:
        """绝对坐标归一化"""
        self.x /= size.width
        self.y /= size.height

    def normalize(self, size: PSize2D) -> Self:
        """获取绝对坐标归一化"""
        p = copy(self)
        p.normalize_me(size)
        return p

    def is_normalized(self) -> bool:
        """判断坐标是否被归一化"""
        return is_normalized(self.x) and is_normalized(self.y)

    def absolutize_me(self, size: PSize2D) -> None:
        """归一化坐标绝对化"""
        self.x = round(self.x * size.width)
        self.y = round(self.y * size.height)

    def absolutize(self, size: PSize2D) -> Self:
        """获取归一化坐标绝对化"""
        p = copy(self)
        p.absolutize_me(size)
        return p

    def to_tuple(self) -> tuple[float, float]:
        """获取tuple"""
        return self.x, self.y

    def to_tuple_int(self) -> tuple[int, int]:
        """获取tuple"""
        return int(self.x), int(self.y)

    def dist2(self, p: Self) -> float:
        """获取点到点距离平方"""
        dx = self.x - p.x
        dy = self.y - p.y
        return dx * dx + dy * dy

    def dist(self, p: Self) -> float:
        """获取点到点距离"""
        return math.sqrt(self.dist2(p))

    def __add__(self, p: Self) -> Self:
        """点相加"""
        return Point(x=self.x + p.x, y=self.y + p.y)

    def __sub__(self, p: Self) -> Self:
        """点相减"""
        return Point(x=self.x - p.x, y=self.y - p.y)

    def __mul__(self, p: Self) -> Self:
        """点相乘"""
        return Point(x=self.x * p.x, y=self.y * p.y)

    def dist_polygon(self, polygon: Sequence[Self]) -> float:
        """点到多边形距离, 返回值: 正数-点在多边形**内部**, 0-点在多边形边上, 负数-点在多边形**外部**"""
        ps = [p.to_tuple() for p in polygon]
        ps1 = np.asarray(ps, dtype=np.float32)
        d = cv2.pointPolygonTest(ps1, self.to_tuple(), True)
        assert isinstance(d, float)
        return d

    def inside(self, polygon: Sequence[Self]) -> bool:
        """判定点是否在多边形内部"""
        return self.dist_polygon(polygon) > 0

    def outside(self, polygon: Sequence[Self]) -> bool:
        """判定点是否在多边形内部"""
        return self.dist_polygon(polygon) < 0


type Points = list[Point]
"""点集定义"""


def rect_vertexes(x1: float, y1: float, x2: float, y2: float) -> Points:
    """生成矩形顶点点集"""
    return [Point(x=x1, y=y1), Point(x=x2, y=y1), Point(x=x2, y=y2), Point(x=x1, y=y2)]


class Normalized(Protocol):
    def is_normalized(self) -> bool:
        pass


def array_is_normalized(arr: Iterable[Normalized]) -> bool:
    """判断数组是否归一化"""
    for e in arr:
        if not e.is_normalized():
            return False
    return True


def array_absolutize(points: Iterable[Point], size: PSize2D) -> Points:
    """数组元素绝对化"""
    return [e.absolutize(size) for e in points]


def array_normalize(points: Iterable[Point], size: PSize2D, offset: Point = Point()) -> Points:
    """数组元素归一化"""
    return [(p + offset).normalize(size) for p in points]


def closest_point(point: Point, points: Points) -> tuple[float, Point]:
    """获取点到点集的最近距离"""
    assert len(points) > 0
    p1 = points[0]
    d2_min = point.dist2(p1)
    for p in points:
        d2 = point.dist2(p)
        if d2 < d2_min:
            d2_min = d2
            p1 = p

    return d2_min, p1


def insert_closest(arr: Points, p: Point, new_p: Point) -> None:
    """在点集中p点前或后插入点，距离p的前一个点近，则插入p前，否则插入p后"""
    assert len(arr) > 0
    i = arr.index(p)
    i2 = (i + 1) % len(arr)
    d1 = new_p.dist2(arr[i - 1])
    d2 = new_p.dist2(arr[i2])
    pos = i if d1 < d2 else i2
    arr.insert(pos, new_p)
